#pragma strict_types

inherit "/obj/xthing";

#define DEBUG

#include "/players/sam/lib/std.h"
#include "/players/sam/lib/hex.h"
#include "/players/sam/lib/fmgr.h"
#include "/system/sendhex.h"

/***** Global variables *****/
static int switch_val;
static string graph_str;
static object owner;
static string owner_name;
static int owned;
static int on_floor;
static mixed objdesc;
static int computing;

/***** Function prototypes *****/
static void draw_it();
static void draw_new();

/*********************************************************************
***** Init and Reset Functions                                   *****
*********************************************************************/ 
void init() 
{
  add_action ("info", "help");
  add_action ("new_move", "micro_w");
  add_action ("new_move", "micro_e");
  add_action ("new_move", "micro_s");
  add_action ("new_move", "micro_n");
  add_action ("switch_mode", "switch");
  add_action ("load_image", "image");
  add_action ("change_pict", "scale");
  add_action ("change_pict", "transl");
  add_action ("change_pict", "flip");
  add_action ("free", "free");
  add_action ("save", "saveimg");
}

void reset(int arg) 
{
  ::reset(arg);
  if(arg)return;

  get_unique_idstr();
  graph_str = 
    "07040F7F7F778C878C878C0401807F778C777F777F04017F7F878C877F877F04"+
    "0A088F088CF68CF68F040A08E8088C0A8C0AE8040AF4E9F48EF68EF6E9040A08"+
    "E908E5F5E5F5E9";
  switch_val = 0;
  on_floor = 0;
  owned = 0;
  objdesc = ({ });
  computing = 0;
}

/*********************************************************************
***** Drop and Get functions                                     *****
*********************************************************************/
int get() 
{
  if (owned && this_player()!=owner) {
    notify_fail("It belongs to "+owner_name+"\n");
    return 0;
  } else {
    owner = this_player();
    owner_name = (string) owner->query_name();
    /* get() seems to be called two times during clone operation */
    if (!owned)
      say("ObjTool now belongs to "+owner_name+"\n");
    owned = 1;
  }
  on_floor = 0;
  return 1;
}

status drop() 
{
  if (!owned || this_player()!=owner) return 1;
  
  on_floor = 1;
  return 0;
}

/*********************************************************************
***** Description Functions                                      *****
*********************************************************************/
int id(string str) 
{
 return(str=="ObjTool" || str=="objtool");
}

string short() 
{
  if (owned) return(owner_name+"'s ObjTool");
  else return("A ObjTool without proprietor");   
}

void long(string str)
{
  write("This ObjTool allows you to manipulate object picture.\n"+
        "Use 'help objtool' if you want do know more about it\n");
}

int info(string helpstr) 
{
  string str, topic;
  
  if (helpstr == "") return 0;

  if (sscanf(helpstr, "%s %s", str, topic) != 2) {
    str = helpstr;
    topic = "";
  }

  if (!id(str)) return 0;

  if (topic == "") {
    write
      ("            Little Tools make Great Things\n" +
       "\n"+
       "This tool allows you to choose the best place and the best dimension\n"+
       "of your objects.\n"+
       "\n"+
       "Usage: - image <object>.....transforms the tool into 'object'.\n"+
       "       - switch.............moves ObjTool with cursor keys.\n"+
       "       - scale <x> [<y>]....scales the picture.\n"+
       "       - transl <x> [<y>]...translates the picture.\n"+
       "       - flip <h|v>.........flips the picture.\n"+
       "       - saveimg............saves the current pict.\n"+
       "\n"+
       "Even if you drop ObjTool, it belongs you. So nobody can play\n"+
       "with it. Use <free> to release ObjTool.\n"+
       " \nWant more help ? Use 'help objtool <command>'." +
       " \nFound some bugs ? Keep cool, and mail to Sam...\n"
       );
  } else {
    switch (topic) {
    case "image" :
      write("                   ObjTool Help\n" +
	    " \n" +
	    "Syntax: image <obj>\n" +
	    " \n" +
	    "Loads the picture of an object.\n" +
	    " \n" +
	    "<obj> can either be an object name (in the current room),\n" +
	    "a player name, or an object file name.\n"
	    );
      break;
    case "switch" :
      write("                   ObjTool Help\n" +
	    " \n" +
	    "Syntax: switch\n" +
	    " \n" +
	    "Switches the cursor keys to move ObjTool, instead of your own\n" +
	    "picture. The screen's location of ObjTool is printed.\n" +
	    "'switch' again to retrieve your character control.\n" +
	    " \n" +
	    "This command is usefull to find the best place of an object.\n"
	    );
      break;
    case "scale" :
      write("                   ObjTool Help\n" +
	    " \n" +
	    "Syntax: scale <xs> [<ys>]\n" +
	    " \n" +
	    "Scales the picture by <xs> percent in X direction, and\n" +
	    "<ys> percent in Y direction.\n" +
	    " \n" +
	    "If not specified, <ys> will be the same as <xs>.\n" +
	    "<xs> and <ys> must be > 0\n"
	  );
      break;
    case "transl" :
      write("                   ObjTool Help\n" +
	    " \n" +
	    "Syntax: transl <xt> [<yt>]\n" +
	    " \n" +
	    "Translates the picture by <xt> pixels in X direction, and\n" +
	    "<yt> pixles in Y direction.\n" +
	    " \n" +
	    "If not specified, <yt> will be 0.\n" +
	    "<xs> and <ys> could be < 0\n"
	  );
      break;
    case "flip" :
      write("                   ObjTool Help\n" +
	    " \n" +
	    "Syntax: flip <h|v>\n" +
	    " \n" +
	    "Flips the picture horizontally (h), or vertically (v).\n"
	  );
      break;
    case "saveimg" :
      write("                   ObjTool Help\n" +
	    " \n" +
	    "Syntax: saveimg\n" +
	    " \n" +
	    "Saves the current picture in ~/objtool.bsx\n" +
	    "The format used in the same as the .bsx format from 'muddraw',\n" +
	    "i.e. a simple hexadecimal description.\n"
	  );
      break;
    case "free" :
      write("                   ObjTool Help\n" +
	    " \n" +
	    "Syntax: free\n" +
	    " \n" +
	    "Releases ObjTool so that an other player can get it.\n" +
	    " \n" +
	    "Your ObjTool cannot be used by another player, even if you\n" +
	    "dropped it, until you release it.\n"
	  );
      break;
    }
  }
  return 1;
}

string graph() 
{
  return(graph_str);
}

/*********************************************************************
***** General 'Local' functions                                  *****
*********************************************************************/
int new_move()                      /***** Handle ObjTool moves *****/
{
  if (!owned || this_player()!=owner) return 0;

  if (!switch_val) return 0;

  switch (query_verb()) {
  case "micro_w":
    if (pos > 0) pos = pos - 1;
    break;
  case "micro_e":
    if (pos < 15) pos = pos + 1;
    break;
  case "micro_n":
    if (depth < 7) depth = depth + 1;
    break;
  case "micro_s":
    if (depth > 0) depth = depth - 1;
    break;
  }

  draw_it();

  return 1;
}

int load_image(string str)          /***** Define the picture to use *****/
{
  string filen, qid;
  object obj;

  if (!owned || this_player()!=owner) return 0;

  if (!on_floor) {
    notify_fail("Drop ObjTool before to change the image\n");
    return 0;
  }

  if (computing == 1) {
    notify_fail("ObjTool is always working for you. Try later...\n");
    return 0;
  }

  /***** Search the picture  *****/
  /*** Test if it is a player or an object ***/
  obj = find_living(str);
  if (!obj) obj = present(str);
  if (!obj) obj = present(str, this_player());

  if (obj) {
    qid = (string) obj->query_idstr();
    if (!qid) graph_str = (string) obj->graph();
    else graph_str = (string) qid->graph();
    
    if (graph_str == "00") {
      notify_fail("(ObjTool) Error: No picture found in that object\n");
      return 0;
    }
  } else {
  /*** Test if it is a file name ***/
    filen = findfile(str);
    if (filen == "") {
      notify_fail("(ObjTool) Error : Wrong object or file name\n");
      return 0;
    }

    qid = (string) filen->query_idstr();
    if (!qid) { qid = filen; }
    graph_str = (string) qid->graph();

    if (graph_str == "00") {
      notify_fail("(ObjTool) Error: No picture found in that object\n");
      return 0;
    }
  }

  draw_new();

  /***** Run the polygons extractor *****/
  computing = 1;
  call_out("poly_extr", 1);
  write("(ObjTool) Polygon Extractor launched....\n");

  return 1;
}

int switch_mode ()                    /***** Switch micro_moves *****/
{
  if (!owned || this_player()!=owner) return 0;

  if (!on_floor)
   {
    write("Drop it before !\n");
    return 1;
   }
  if (!switch_val) {
    write ("(ObjTool) Now the keypad moves ObjTool\n");
    switch_val = 1;
  } else {
    write ("(ObjTool) Now, you move yourself\n");
    switch_val = 0;
  }
  return 1;
}

int change_pict(string arg)         /***** Launch a transformation *****/
{
  if (!owned || this_player()!=owner) return 0;

  if (computing == 1) {
    notify_fail("ObjTool is always working for you. Try later...\n");
    return 0;
  }

  if (arg == "") {
    notify_fail("(ObjTool) Error : This function needs an argument\n");
    return 0;
  }

  /***** Run the function *****/
  computing = 1;
  switch(query_verb()) {
  case "scale":
    call_out("do_scale", 1, arg);
    break;
  case "transl":
    call_out("do_trans", 1, arg);
    break;
  case "flip":
    call_out("do_flip", 1, arg);
    break;
  }

  write("(ObjTool) Computation launched, please wait a bit...\n");

  return 1;
}

int save()                          /***** Save current drawing *****/
{
  string filen;

  if (!owned || this_player()!=owner) return 0;
  
  filen = "/players/" + lower_case((string)(owner->query_real_name())) +
          "/objtool.bsx";
  if (file_size(filen) != -1) { 
    if (!rm(filen)) {
      notify_fail("(ObjTool) Error: Unable to remove old backup file\n");
      return 0;
    }
  }
    
  if (!write_file(filen, graph_str)) {
    notify_fail("(ObjTool) Error: Unable to create the backup file\n");
    return 0;
  }

  write ("(ObjTool) Your graphic is saved in '~/objtool.bsx'\n");
  return 1;
}

int free(string str)                     /***** Release ObjTool *****/
{
  if (!owned || this_player()!=owner) return 0;
  
  if (!on_floor) {
    write ("You have to drop it before !\n");
    return 1;
  } else {
    say (owner_name+" releases ObjTool.\n" +
	 "Anyone can get it now.\n");
    owned = 0;
    return 1;
  }
}

/*********************************************************************
***** Internal Functions                                         *****
*********************************************************************/
static void draw_it()          /***** Really change the picture *****/
{
  gfx_tell_room(environment(this_player()),
        "@"+"RMO"+idstr+"."+
        "@"+"VIO"+idstr+"."+sendhex(pos)+sendhex(depth)+
        "@"+"RFS");
  write("(ObjTool) Position: "+pos+" - "+depth+"\n");
}

static void draw_new()               /***** Draw a new picture *****/
{
  gfx_tell_room(environment(this_player()),
		"@"+"RMO"+idstr+". ");
  gfx_tell_room(environment(this_player()),
		"@"+"DFO"+idstr+"."+graph_str+" ");
  gfx_tell_room(environment(this_player()),
		"@"+"VIO"+idstr+"."+sendhex(pos)+sendhex(depth)+" ");
  gfx_tell_room(environment(this_player()),
		"@"+"RFS");
}

/*********************************************************************
***** 'Background' computations : Polygon Extraction             *****
*********************************************************************/ 
static void poly_extr()
{
  int nb_poly;
  int nb_vert;
  int index;
  int i, j;

  nb_poly = readhex(graph_str[0..1]);
  objdesc = allocate(nb_poly+1);
  write(" \n(ObjTool) Report: Found "+nb_poly+" polygons.\n");

  /***** Reset bounding box (in objdes[0][0..3]) *****/
  objdesc[0] = ({0,0,0,0}); /* width,height,xmin,ymin */

  /***** Extract each polygons *****/
  index = 2;
  for (i=1; i<=nb_poly; i++) {
    nb_vert = readhex(graph_str[index..index+1]);
    index += 2;
    objdesc[i] = allocate((2*nb_vert)+1);

    /** Color extraction **/
    objdesc[i][0] = readhex(graph_str[index..index+1]);
    index += 2;

    /** Vertices extraction **/
    for(j=0; j<nb_vert; j++) {
      objdesc[i][(2*j)+1] = (readhex(graph_str[index..index+1])-127) * 1024;
      index += 2;
      objdesc[i][(2*j)+2] = (readhex(graph_str[index..index+1])-127) * 1024;
      index += 2;
    }
  } 

  write("(ObjTool) Report: Polygons extracted.\n"+
	"                  Bounding box computation launched...\n");

  call_out("do_bb", 1);
}

void do_bb()
{
  int xmin, xmax;
  int ymin, ymax;
  int val;
  int i, j;

  xmin = 255 * 1024;
  xmax = -255 * 1024;
  ymin = 255 * 1024;
  ymax = -255 * 1024;

  /***** Compute Bounding Box *****/
  for(i=1; i<(sizeof(objdesc)); i++) {
    for (j=0; j<(sizeof(objdesc[i])-1)/2; j++) {
      val = objdesc[i][(2*j)+1]; /* X */
      if (val < xmin) xmin = val;
      if (val > xmax) xmax = val;

      val = objdesc[i][(2*j)+2]; /* Y */
      if (val < ymin) ymin = val;
      if (val > ymax) ymax = val;
    }
  }

  objdesc[0][0] = xmax-xmin;
  objdesc[0][1] = ymax-ymin;
  objdesc[0][2] = xmin;
  objdesc[0][3] = ymin;

  write("(ObjTool) Report: Bounding Box : w = " + objdesc[0][0]/1024 +
        " / h = " + objdesc[0][1]/1024 +
	" - coord = " + xmin/1024 + "/" + ymin/1024 + "\n" +
	"                  Done.....\n");

  computing = 0;
}

/*********************************************************************
***** 'Background' computations : Scaling                        *****
*********************************************************************/ 
void do_scale(string arg) 
{
  int xs, ys;
  int xmin, ymin;
  int width, height;
  int xtrans, ytrans;
  int val;
  int i, j;
  int x, y;

  /***** Extract scaling factors *****/
  if (sscanf(arg, "%d %d", xs, ys) != 2) {
    if (sscanf(arg, "%d", xs) != 1) {
      write("(ObjTool) Error found in the function arguments..\n"+
	    "          Computation killed...\n");
      computing = 0;
      return;
    }
    ys = xs;
  }

  if (xs<=0 || ys<=0) {
    write("(ObjTool) Error : Arguments should be > 0\n"+
	  "          Computation killed...\n");
    computing = 0;
    return;
  }

  /***** Get BBox *****/
  width = objdesc[0][0];
  height = objdesc[0][1];
  xmin = objdesc[0][2];
  ymin = objdesc[0][3];

  /***** Check the scale *****/
  if ( (height*ys)/100 > (255<<10) ) {
    write("(ObjTool) Error : Scale is too big in Y\n"+
	  "          Maximum scale is : " + ((255<<10)*100)/height + "\n" +
	  "          Computation killed...\n");
    computing = 0;
    return;
  }

  if ( (width*xs)/100 > (255<<10) ) {
    write("(ObjTool) Error : Scale is too big in X\n"+
	  "          Maximum scale is : " + ((255<<10)*100)/width + "\n" +
	  "          Computation killed...\n");
    computing = 0;
    return;
  }

  /***** Check if translation is needed after scaling *****/
  val =  ((width*xs)+(xmin*100))/100;
  if (val > (127<<10)) xtrans = (127<<10) - val;

  val =  ((height*ys)+(ymin*100))/100;
  if (val > (127<<10)) ytrans = (127<<10) - val;
  

  /****** Compute the scaled polygones ******/
  for(i=1; i<(sizeof(objdesc)); i++) {
    for (j=0; j<(sizeof(objdesc[i])-1)/2; j++) {
      x = objdesc[i][(2*j)+1]; /* X */
      x = ((((x-xmin)*xs)+(xmin*100))/100) + xtrans;
      objdesc[i][(2*j)+1] = x;

      y = objdesc[i][(2*j)+2]; /* Y */
      y = ((((y-ymin)*ys)+(ymin*100))/100) + ytrans;
      objdesc[i][(2*j)+2] = y;
    }
  }

  /***** Compute the new bounding box *****/
  objdesc[0][0] = (width*xs)/100;
  objdesc[0][1] = (height*ys)/100;
  objdesc[0][2] = xmin + xtrans;
  objdesc[0][3] = ymin + ytrans;

  /***** Launch new pict computation *****/
  write(" \n(ObjTool) Report : Scaling done\n" +
        "                   Launch new pict computation...\n");

  call_out("compute_pict", 1);
}

/*********************************************************************
***** 'Background' computations : Translating                    *****
*********************************************************************/ 
void do_trans(string arg) 
{
  int xt, yt;
  int xmin, ymin;
  int width, height;
  int i, j;
  int x, y;

  /***** Extract translating factors *****/
  if (sscanf(arg, "%d %d", xt, yt) != 2) {
    if (sscanf(arg, "%d", xt) != 1) {
      write("(ObjTool) Error found in the function arguments..\n"+
	    "          Computation killed...\n");
      computing = 0;
      return;
    }
    yt = 0;
  }

  xt = xt * 1024;
  yt = yt * 1024;

  /***** Get BBox *****/
  width = objdesc[0][0];
  height = objdesc[0][1];
  xmin = objdesc[0][2];
  ymin = objdesc[0][3];

  /***** Check the translation *****/
  if (xt < 0) {
    if ( (xmin+xt) < (-127 * 1024) ) {
      write("(ObjTool) Error : Translation is too big in -X\n"+
	 "          Maximum value is : "+ (((-127 * 1024)-xmin)/1024) +"\n" +
	    "          Computation killed...\n");
      computing = 0;
      return;
    }
  } else {
    if ( (xmin+width+xt) > (127*1024) ) {
      write("(ObjTool) Error : Translation is too big in +X\n"+
	"          Maximum value is : "+ (((127*1024)-xmin-width)/1024) + "\n" +
	    "          Computation killed...\n");
      computing = 0;
      return;
    }
  }

  if (yt < 0) {
    if ( (ymin+yt) < (-127 * 1024) ) {
      write("(ObjTool) Error : Translation is too big in -Y\n"+
	 "          Maximum value is : " + (((-127 * 1024)-ymin)/1024) + "\n" +
	    "          Computation killed...\n");
      computing = 0;
      return;
    }
  } else {
    if ( (ymin+height+yt) > (127*1024) ) {
      write("(ObjTool) Error : Translation is too big in +Y\n"+
       "          Maximum value is : "+ (((127*1024)-ymin-height)/1024) + "\n" +
	    "          Computation killed...\n");
      computing = 0;
      return;
    }
  }

  /****** Compute the translated polygones ******/
  for(i=1; i<(sizeof(objdesc)); i++) {
    for (j=0; j<(sizeof(objdesc[i])-1)/2; j++) {
      x = objdesc[i][(2*j)+1]; /* X */
      x += xt;
      objdesc[i][(2*j)+1] = x;

      y = objdesc[i][(2*j)+2]; /* Y */
      y += yt;
      objdesc[i][(2*j)+2] = y;
    }
  }

  /***** Compute the new bounding box *****/
  objdesc[0][2] = xmin + xt;
  objdesc[0][3] = ymin + yt;

  /***** Launch new pict computation *****/
  write(" \n(ObjTool) Report : Translation done\n" +
        "                   Launch new pict computation...\n");

  call_out("compute_pict", 1);
}

/*********************************************************************
***** 'Background' computations : Flipping                       *****
*********************************************************************/ 
void do_flip(string arg) 
{
  string direction;
  int xmin, ymin;
  int width, height;
  int xmax, ymax;
  int i, j;
  int x, y;

  /***** Extract translating factors *****/
  if (sscanf(arg, "%s", direction) != 1) {
    if (direction != "h" && direction != "v") {
      write("(ObjTool) Error found in the function arguments..\n"+
	    "          Computation killed...\n");
      computing = 0;
      return;
    }
  }

  /***** Get BBox *****/
  width = objdesc[0][0];
  height = objdesc[0][1];
  xmin = objdesc[0][2];
  ymin = objdesc[0][3];
  xmax = xmin+width;
  ymax = ymin+height;

  /****** Compute the flipped polygones ******/
  switch (direction) {
  case "h" :
    for(i=1; i<(sizeof(objdesc)); i++) {
      for (j=0; j<(sizeof(objdesc[i])-1)/2; j++) {
	x = objdesc[i][(2*j)+1]; /* X */
	x = xmax - (x-xmin);
	objdesc[i][(2*j)+1] = x;
      }
    }
    break;

  case "v" :
    for(i=1; i<(sizeof(objdesc)); i++) {
      for (j=0; j<(sizeof(objdesc[i])-1)/2; j++) {
	y = objdesc[i][(2*j)+2]; /* Y */
	y = ymax - (y-ymin);
	objdesc[i][(2*j)+2] = y;
      }
    }
    break;
  }

  /***** Launch new pict computation *****/
  write(" \n(ObjTool) Report : Flipping done\n" +
        "                   Launch new pict computation...\n");

  call_out("compute_pict", 1);
}

/*********************************************************************
***** 'Background' computations : New picture                    *****
*********************************************************************/ 
void compute_pict()
{
  int i, j;

  /***** Create the graph string *****/
  graph_str = writehex(sizeof(objdesc)-1);

  for(i=1; i<(sizeof(objdesc)); i++) {

    graph_str += writehex((sizeof(objdesc[i])-1)/2) + writehex(objdesc[i][0]);

    for (j=0; j<(sizeof(objdesc[i])-1)/2; j++) {
      graph_str += writehex(objdesc[i][(2*j)+1]/1024 + 127) + 
	           writehex(objdesc[i][(2*j)+2]/1024 + 127);
    }
  }
    
  write("(ObjTool) Report: New Bounding Box : w = " + objdesc[0][0]/1024 +
        " / h = " + objdesc[0][1]/1024 +
	" - coord = " + objdesc[0][2]/1024 + "/" + objdesc[0][3]/1024 + "\n" +
	"                  Done.....\n");

  draw_new();

  computing = 0;
}
