#include "/players/wing/std.h"

#define DRAWING_NAME 0
#define DRAWING_POLYGONS 1

mixed drawings;
static int current;
static int mode;
static object room;
static int serverp;
static int background_color = 7;

#define DRAWING_SERVER "/players/wing/drawing_server"

#define MODE_NORMAL 0
#define MODE_ROOM 1
#define MODE_PAPER 2

#define COLORS ({ "black", "blue", "green", "skyblue", "red", \
		  "pink", "brown", "lightgrey", "grey", "lightblue", \
		  "lightgreen", "cyan", "darkred", "mangenta", "yellow", \
		  "white" })

status create_room();
      
void reset( int flag )
{
  string a;
  
  if (flag)
    return;
  if (sscanf( FNTO, "%s#%s", a, a ) != 2)
    return;
  drawings = ({ });
  if (TP->query_level() >= 21
      && restore_object( "players/" + TP->query_real_name() + "/drawings" ) );
  else
  {
    drawings = DRAWING_SERVER->get_drawings( TP->query_real_name() );
    serverp = 1;
  }
  current = -1;
}

void query_auto_load()
{
  return explode( FN( TO ), "#" )[0] + ":";
}

void init()
{
  add_action( "help", "help" );
  add_action( "save", "save" );
  add_action( "list", "list" );
  add_action( "rename", "rename" );
  add_action( "check", "check" );
  add_action( "check2", "check2" );
  add_action( "colors", "colors" );
  add_action( "name", "name" );
  add_action( "work", "work" );
  add_action( "steal", "steal" );
  add_action( "remove", "remove" );
  add_action( "draw", "draw" );
  add_action( "change", "change" );
  add_action( "move_polygon", "move" );
  add_action( "copy", "copy" );
  add_action( "insert", "insert" );
  add_action( "erase", "erase" );
  add_action( "get_string", "string" );
  if (TP->query_level() >= 21)
    add_action( "create_something", "create" );
  add_action( "bounding_box", "bounding" );
  add_action( "mode", "mode" );
  /*  add_action( "set", "set" );*/
  add_action( "background", "background" );
  add_action( "flip", "flip" );
}

#define SAVE_HELP	"save drawings\n"
#define LIST_HELP	"list drawings\n"
#define RENAME_HELP	"rename [drawing] [new name]\n"
#define CHECK_HELP	"check [drawing]\n"
#define CHECK2_HELP	"check2 [drawing]\n"
#define COLORS_HELP	"colors\n"
#define NAME_HELP	"name [new drawing]\n"
#define WORK_HELP	"work [drawing]\n"
#define STEAL_HELP	"steal [object] as [name]\n"
#define REMOVE_HELP	"remove [drawing]\n"
#define DRAW_HELP	"draw [color] [x0] [y0] ... [xn] [yn]\n"
#define CHANGE_HELP	"change [polygon] to [color]\n" + \
			"change [polygon] to [x0] [y0] ... [xn] [yn]\n"
#define MOVE_HELP	"move [polygon[-polygon]] [delta x] [delta y]\n"
#define COPY_HELP	"copy [polygon[-polygon]] [delta x] [delta y]\n"
#define INSERT_HELP	"insert [color] [x0] [y0] ... [xn] [yn] at [polygon]\n"
#define ERASE_HELP	"erase [polygon]\n"
#define STRING_HELP	"string [drawing]\n"
#define CREATE_OBJECT_HELP	"create object\n"
#define BOUNDING_BOX_HELP	"bounding box [drawing]\n"
#define MODE_HELP	"mode normal|paper\n"
#define BACKGROUND_HELP	"background [color]\n"
#define FLIP_HELP	"flip horisontal|vertical\n"
			

#define ALL_HELP SAVE_HELP + LIST_HELP + RENAME_HELP +  CHECK_HELP + \
		 CHECK2_HELP + COLORS_HELP +  NAME_HELP +  WORK_HELP + \
		 STEAL_HELP + REMOVE_HELP + DRAW_HELP + CHANGE_HELP + \
		 MOVE_HELP + COPY_HELP + INSERT_HELP + ERASE_HELP + \
		 STRING_HELP + \
		 (TP->query_level() >= 21 ? CREATE_OBJECT_HELP : "") + \
		 BOUNDING_BOX_HELP + MODE_HELP + BACKGROUND_HELP + \
		 FLIP_HELP

status help( string str )
{
  if (stringp( str ) && str == "pen" )
  {
    my_write( "With this magic pen you can do the following:\n" + ALL_HELP );
    return 1;
  }
  return 0;
}
	      
status save( string str )
{
  if (!stringp( str ) || lower_case( str ) != "drawings" )
    return 0;
  if (TP->query_level() >= 21)
  {
    if (serverp)
      DRAWING_SERVER->remove_drawings( TP->query_real_name() );
    save_object( "players/" + TP->query_real_name() + "/drawings" );
  }
  else
    DRAWING_SERVER->put_drawings( TP->query_real_name(), drawings );
}

status list( string str )
{
  int c;

  if (!stringp( str ) || lower_case( str ) != "drawings" )
    return 0;
  for (c=0; c < sizeof( drawings ); c++)
    my_write( (c == current ? "*" : "" ) + c + ": " +
	      drawings[c][ DRAWING_NAME ] + "\n" );
  return 1;
}

int color_number( string color_name )
{
  int c;
  
  if (!stringp( color_name ))
    return -1;
  color_name = lower_case( color_name );
  for (c=0; c < 16; c++)
    if (COLORS[c] == color_name)
      return c;
  return -1;
}

string color_name( int color_number )
{
  if (color_number < 0 || color_number >= 16)
    return "invalid";
  return COLORS[ color_number ];
}

int find_drawing( string name )
{
  int c;

  for (c=0; c < sizeof( drawings ); c++)
    if (drawings[c][DRAWING_NAME] == name)
      return c;
  return -1;
}

void change_current( int to )
{
  if (current != to && to != -1 && mode)
  {
    current = to;
    create_room();
  }
  else
    current = to;
}

status rename( string str )
{
  string new_name, old_name;
  int number, tmp;

  if (!stringp( str ))
    return 0;
  tmp = sscanf( str, "%d %s", number, new_name );
  if (tmp == 0 && sscanf( str, "%s %s", old_name, new_name ) == 2)
  {
    tmp = 2;
    number = find_drawing( old_name ); 
    if (number == -1)
    {
      my_notify_fail( "No drawing named " + old_name + ".\n" );
      return 0;
    }
  }
  if (tmp == 2)
  {
    if (number >= sizeof( drawings ))
    {
      my_notify_fail( "There are not that many drawings.\n" );
      return 0;
    }
    my_write( "Drawing " + drawings[number][DRAWING_NAME] + " renamed to "
	      + new_name + ".\n" );
    drawings[number][DRAWING_NAME] = new_name;
    return 1;
  }
  else
  {
    my_notify_fail( "Usage: " + RENAME_HELP );
    return 0;
  }
}

status check( string str )
{
  int number, c, c1;
  string tmp;
  
  if (!stringp( str ) || str == "")
  {
    if (current != -1)
      number = current;
    else
    {
      my_notify_fail( "You are not working on any drawing.\n" );
      return 0;
    }
  }
  else if (sscanf( str, "%d", number ) != 1)
  {
    number = find_drawing( str );
    if (number == -1)
    {
      my_notify_fail( "No drawing is named " + str + ".\n" );
      return 0;
    }
  }
  if (number > sizeof( drawings ) - 1)
  {
    my_notify_fail( "There are not that many drawings.\n" );
    return 0;
  }
  for (c=0; c < sizeof( drawings[number][DRAWING_POLYGONS] ); c++)
  {
    tmp = c + ": " + color_name( drawings[number][DRAWING_POLYGONS][c][0] ) +
      " (" + drawings[number][DRAWING_POLYGONS][c][1] + "," +
      drawings[number][DRAWING_POLYGONS][c][2] + ")";
    for (c1=3; c1 < sizeof( drawings[number][DRAWING_POLYGONS][c] ); c1+=2)
      tmp += "-(" + drawings[number][DRAWING_POLYGONS][c][c1] + "," +
	drawings[number][DRAWING_POLYGONS][c][c1+1] + ")";
    my_write( tmp + "\n" );
  }
  change_current( number );
  return 1;
}

status check2( string str )
{
  int number, c, c1;
  string tmp;
  
  if (!stringp( str ) || str == "")
  {
    if (current != -1)
      number = current;
    else
    {
      my_notify_fail( "You are not working on any drawing.\n" );
      return 0;
    }
  }
  else if (sscanf( str, "%d", number ) != 1)
  {
    number = find_drawing( str );
    if (number == -1)
    {
      my_notify_fail( "No drawing is named " + str + ".\n" );
      return 0;
    }
  }
  if (number > sizeof( drawings ) - 1)
  {
    my_notify_fail( "There are not that many drawings.\n" );
    return 0;
  }
  for (c=0; c < sizeof( drawings[number][DRAWING_POLYGONS] ); c++)
  {
    tmp = c + ": " + color_name( drawings[number][DRAWING_POLYGONS][c][0] ) +
      " " + drawings[number][DRAWING_POLYGONS][c][1] + " " +
      drawings[number][DRAWING_POLYGONS][c][2];
    for (c1=3; c1 < sizeof( drawings[number][DRAWING_POLYGONS][c] ); c1+=2)
      tmp += " " + drawings[number][DRAWING_POLYGONS][c][c1] + " " +
	drawings[number][DRAWING_POLYGONS][c][c1+1];
    my_write( tmp + "\n" );
  }
  change_current( number );
  return 1;
}

status colors( string str )
{
  int c;
  
  for (c=0; c < 16; c++)
    my_write( c + ": " + COLORS[c] + "\n" );
  return 1;
}

status name( string str )
{
  string tmp;
  
  if (!stringp( str ))
    return 0;
  drawings += ({ ({ str, ({ }) }) });
  change_current( sizeof( drawings ) - 1 );
  my_write( "You decide name your new drawing " + str + ".\n" );
  return 1;
}

status work( string str )
{
  int number;
  string tmp;
  
  if (!stringp( str ))
    return 0;
  if (sscanf( str, "%d %s", number, tmp ) == 1)
  {
    if (number < 0 || number >= sizeof( drawings ))
    {
      my_notify_fail( "There are not that many drawings.\n" );
      return 0;
    }
  }
  else if( (number = find_drawing( str )) == -1)
  {
    my_notify_fail( "There is no drawing named " + str + ".\n" );
    return 0;
  }
  my_write( "You decide to start working on the drawing " +
	   drawings[number][DRAWING_NAME] + ".\n" );
  change_current( number );
  return 1;
}

#define CHECK_RANGE(a) \
if (c+a > sizeof( intstream )) \
{ \
  my_notify_fail( "Corrupt graphics data in object.\n" );\
  return 0;\
}
		  
status steal( string str )
{
  mixed obj;
  string name;
  int *intstream, total_size, polygon_size, *polygon, c, c1, c2;
  mixed polygons;

  if (!stringp( str ) || sscanf( str, "%s as %s", str, name ) != 2)
  {
    my_notify_fail( "Usage: " + STEAL_HELP );
    return 0;
  }
  obj = present( str, environment( this_player() ) );
  if (!obj && TP->query_level() >= 21)
    obj = find_object( str );
  if (obj == 0)
  {
    my_notify_fail( "Couldn't find that object.\n" );
    return 0;
  }
  intstream = CONVHEX( obj->graph() );
  if (intstream == 0 || sizeof( intstream ) < 1)
  {
    my_notify_fail( "Corrupt graphics data in object.\n" );
    return 0;
  }
  polygons = ({ });
  total_size = intstream[0];
  c=1;
  for (c1=0; c1 < total_size; c1++)
  {
    CHECK_RANGE(2);
    polygon_size = intstream[c++];
    polygon = ({ intstream[c++] });
    for (c2 = 0; c2 < polygon_size; c2++)
    {
      CHECK_RANGE(2);
      polygon += ({ intstream[c++], intstream[c++] });
    }
    polygons += ({ polygon });
  }
  if (c != sizeof( intstream ))
  {
    my_notify_fail( "Corrupt graphics data in object.\n" );
    return 0;
  }
  drawings += ({ ({ name, polygons }) });
  change_current( sizeof( drawings ) - 1 );
  return 1;
}

status remove( string str )
{
  string tmp;
  int number;
  
  if (!stringp( str ))
    return 0;
  if (sscanf( str, "%d %s", number, tmp ) != 1)
    if ((number = find_drawing( str )) == -1)
    {
      my_notify_fail( "There is no drawing named " + str + ".\n" );
      return 0;
    }
  if (number >= sizeof( drawings ))
  {
    my_notify_fail( "There are not that many drawings.\n" );
    return 0;
  }
  my_write( "You decide to remove the drawing " +
	    drawings[number][DRAWING_NAME] + ".\n" );
  drawings = slice_array( drawings, 0, number - 1 ) +
    slice_array( drawings, number + 1, MYCKE );
  change_current( -1 );
  return 1;
}    

status draw( string str )
{
  int x,y,tmp,color;
  int *polygon;
  string rest;
    
  if (!stringp( str ))
    return 0;
  if (current == -1)
  {
    my_notify_fail( "You are not working on any drawing.\n" );
    return 0;
  }
  if (str == "")
  {
    my_notify_fail( "Usage: " + DRAW_HELP );
    return 0;
  }
  if (sscanf( str, "%d %s", color, rest ) == 2)
  {
    str = rest;
    if (color < 0 || color >= 16)
    {
      my_notify_fail( "The colornumber must be between 0 and 15.\n" );
      return 0;
    }
  }
  else
  {
    if (sscanf( str, "%s %s", rest, str ) != 2)
    {
      my_notify_fail( "Usage: " + DRAW_HELP );
      return 0;
    }
    if ((color = color_number( rest )) == -1)
    {
      my_notify_fail( "There is no color named " + rest + ".\n" );
      return 0;
    }
  }
  polygon = ({ color });
  while ((tmp = sscanf( str, "%d %d %s", x, y, str )) == 3)
    polygon += ({ x, y });
  if (tmp == 2)
    drawings[current][DRAWING_POLYGONS] += ({ polygon + ({ x, y }) });
  else if (tmp == 0)
    drawings[current][DRAWING_POLYGONS] += ({ polygon });
  else
  {
    my_notify_fail( "Usage: " + DRAW_HELP );
    return 0;
  }
  if (mode)
    create_room();
  return 1;
}

status change( string str )
{
  int x,y,tmp,color,number;
  int *polygon;
  string rest;
    
  if (!stringp( str ))
    return 0;
  if (current == -1)
  {
    my_notify_fail( "You are not working on any drawing.\n" );
    return 0;
  }
  if (str == "" || sscanf( str, "%d to %s", number, str ) != 2)
  {
    my_notify_fail( "Usage: change polygon to (color, colornumber or new " +
		    "coordinates.\n" );
    return 0;
  }
  if (number < 0 || number >= sizeof( drawings[current][DRAWING_POLYGONS] ))
  {
    my_notify_fail( "There are not that many plygons.\n" );
    return 0;
  }
  if ((tmp = sscanf( str, "%d %d %s", x, y, rest )) == 0)
  {
    color = color_number( str );
    if (color == -1)
    {
      my_notify_fail( "There is no color named " + str + ".\n" );
      return 0;
    }
  }
  else if (tmp == 1)
  {
    color = x;
    if (color < 0 || color >= 16)
    {
      my_notify_fail( "There are not that many colors.\n" );
      return 0;
    }
  }
  else
  {
    polygon = ({ drawings[current][DRAWING_POLYGONS][number][0], x, y });
    str = rest;
    while ((tmp = sscanf( str, "%d %d %s", x, y, str )) == 3)
      polygon += ({ x, y });
    if (tmp == 2)
      drawings[current][DRAWING_POLYGONS][number] = polygon + ({ x, y });
    else if (tmp == 0)
      drawings[current][DRAWING_POLYGONS][number] = polygon;
    else
    {
      my_notify_fail( "Usage: " + CHANGE_HELP );
      return 0;
    }
    if (mode)
      create_room();
    return 1;
  }
  drawings[current][DRAWING_POLYGONS][number][0] = color;
  if (mode)
    create_room();
  return 1;
}

status copy( string str )
{
  int x, y, number, number2, c, c1, tmp;
  mixed *polygons;
  mixed *old_polygons;
    
  if (!stringp( str ))
    return 0;
  if (current == -1)
  {
    my_notify_fail( "You are not working on any drawing.\n" );
    return 0;
  }
  if (str == "" || sscanf( str, "%d %d %d", number, x, y ) != 3
      && sscanf( str, "%d-%d %d %d", number, number2, x, y ) != 4)
  {
    my_notify_fail( "Usage: " + COPY_HELP );
    return 0;
  }
  if (number > number2)
    number2 = number;
  if (number < 0 || number2 > sizeof( drawings[current][DRAWING_POLYGONS] ))
  {
    my_notify_fail( "There are not that many plygons.\n" );
    return 0;
  }
  old_polygons = drawings[ current ][ DRAWING_POLYGONS ][ number..number2 ];
  polygons = allocate( sizeof( old_polygons ) );
  for (c=0; c < sizeof( old_polygons ); c++)
  {
    polygons[c] = allocate( sizeof (old_polygons[c]) );
    polygons[c][0] = old_polygons[c][0];
    for (c1=1; c1 < sizeof( old_polygons[c] ); c1+=2)
    {
      tmp = old_polygons[c][c1] + x;
      if (tmp > 255 || tmp < 0)
      {
	my_notify_fail( "Requested move would move the polygon off the edge " +
		       "of the drawing.\n" );
	return 0;
      }
      polygons[c][c1] = tmp;
      tmp = old_polygons[c][c1+1] + y;
      if (tmp > 255 || tmp < 0)
      {
	my_notify_fail( "Requested move would move the polygon off the edge " +
		       "of the drawing.\n" );
	return 0;
      }
      polygons[c][c1+1] = tmp;
    }
  }
  drawings[current][DRAWING_POLYGONS] += polygons;
  if (mode)
    create_room();
  return 1;
}

status move_polygon( string str )
{
  int x, y, number, number2, c, c1, tmp;
  mixed *polygons;
  mixed *old_polygons;
    
  if (!stringp( str ))
    return 0;
  if (current == -1)
  {
    my_notify_fail( "You are not working on any drawing.\n" );
    return 0;
  }
  if (str == "" || sscanf( str, "%d %d %d", number, x, y ) != 3
      && sscanf( str, "%d-%d %d %d", number, number2, x, y ) != 4)
  {
    my_notify_fail( "Usage: " + MOVE_HELP );
    return 0;
  }
  if (number2 < number)
    number2 = number;
  if (number < 0 || number >= sizeof( drawings[current][DRAWING_POLYGONS] )
      || number2 >= sizeof( drawings[ current][DRAWING_POLYGONS] ))
  {
    my_notify_fail( "There are not that many polygons.\n" );
    return 0;
  }
  old_polygons = drawings[ current ][ DRAWING_POLYGONS ][ number..number2 ];
  polygons = allocate( sizeof( old_polygons ) );
  for (c=0; c < sizeof( old_polygons ); c++)
  {
    polygons[c] = allocate( sizeof( old_polygons[c] ) );
     polygons[c][0] = old_polygons[c][0];
    for (c1=1; c1 < sizeof( old_polygons[c] ); c1+=2)
    {
      tmp = old_polygons[c][c1] + x;
      if (tmp > 255 || tmp < 0)
	FAIL( "Requested move would move the polygon off the edge of the " +
	     "drawing.\n" );
      polygons[c][c1] = tmp;
      tmp = old_polygons[c][c1+1] + y;
      if (tmp > 255 || tmp < 0)
	FAIL( "Requested move would move the polygon off the edge of the " +
	     "drawing.\n" );
      polygons[c][c1+1] = tmp;
    }
  }
  /*  drawings[current][DRAWING_POLYGONS][number..number2] = polygons;
      Bus Errors... tsk, tsk */
  for (c=0; c < sizeof( polygons ); c++)
    drawings[current][DRAWING_POLYGONS][number + c] = polygons[c];
  if (mode)
    create_room();
  return 1;
}

status insert( string str )
{
  string data;
  int color, x, y, *polygon, tmp, number;

  if (current == -1)
  {
    my_notify_fail( "You are not working on any drawing.\n" );
    return 0;
  }
  if (!stringp( str ) || sscanf( str, "%s at %d", data, number ) != 2)
  {
    my_notify_fail( "Usage: " + INSERT_HELP );
    return 0;
  }
  if (sscanf( data, "%d %s", color, data ) == 2)
  {
    if (color < 0 || color >= 16)
    {
      my_notify_fail( "The colornumber must be between 0 and 15.\n" );
      return 0;
    }
  }
  else
  {
    if (sscanf( data, "%s %s", str, data ) != 2)
    {
      my_notify_fail( "Usage: " + DRAW_HELP );
      return 0;
    }
    if ((color = color_number( str )) == -1)
    {
      my_notify_fail( "There is no color named " + str + ".\n" );
      return 0;
    }
  }
  polygon = ({ color });
  while ((tmp = sscanf( data, "%d %d %s", x, y, data )) == 3)
    polygon += ({ x, y });
  if (tmp == 2)
    polygon += ({ x, y });
  else if (tmp != 0)
  {
    my_notify_fail( "Usage: " + INSERT_HELP );
    return 0;
  }
  drawings[current][DRAWING_POLYGONS] =
    slice_array( drawings[current][DRAWING_POLYGONS], 0, number - 1 ) +
    ({ polygon }) +
    slice_array( drawings[current][DRAWING_POLYGONS], number, MYCKE );
  if (mode)
    create_room();
  return 1;
}

status erase( string str )
{
  int number;
    
  if (!stringp( str ))
    return 0;
  if (current == -1)
  {
    my_notify_fail( "You are not working on any drawing.\n" );
    return 0;
  }
  if (str == "" || sscanf( str, "%d", number ) != 1)
  {
    my_notify_fail( "Usage: " + ERASE_HELP );
    return 0;
  }
  if (number < 0 || number > sizeof( drawings[current][DRAWING_POLYGONS] ))
  {
    my_notify_fail( "There are not that many plygons.\n" );
    return 0;
  }
  drawings[current][DRAWING_POLYGONS] =
    slice_array( drawings[current][DRAWING_POLYGONS], 0, number - 1 ) +
    slice_array( drawings[current][DRAWING_POLYGONS], number + 1, MYCKE );
  if (mode)
    create_room();
  return 1;
}

string create_polygon( int *polygon )
{
  string result;
  int c;

  result = SENDHEX( (sizeof( polygon ) - 1) / 2 ); /* size */
  result += SENDHEX( polygon[0] ); /* color */
  for (c=1; c < sizeof( polygon ); c += 2)
    result += SENDHEX( polygon[c] ) + SENDHEX( polygon[c+1] );
  return result;
}

string create_graph( mixed drawing )
{
  string result;
  int c;

  result = SENDHEX( sizeof( drawing ) ); /* Number of polygons */
  for (c=0; c < sizeof( drawing ); c++)
    result += create_polygon( (int *)drawing[c] );
  return result;
}

status get_string( string str )
{
  int number, c, c1;
  string tmp;
  
  if (!stringp( str ) || str == "")
  {
    if (current != -1)
      number = current;
    else
    {
      my_notify_fail( "You are not working on any drawing.\n" );
      return 0;
    }
  }
  else if (sscanf( str, "%d", number ) != 1)
  {
    number = find_drawing( str );
    if (number == -1)
    {
      my_notify_fail( "No drawing is named " + str + ".\n" );
      return 0;
    }
  }
  if (number > sizeof( drawings ) - 1)
  {
    my_notify_fail( "There are not that many drawings.\n" );
    return 0;
  }
  my_write( create_graph( drawings[number][DRAWING_POLYGONS] ) + "\n" );
  change_current( number );
  return 1;
}

status create_object()
{
  object obj;
  
  obj = clone_object( "/players/wing/generic_object" );
  obj->set_graph( create_graph( drawings[current][DRAWING_POLYGONS] ) );
  move_object( obj, TP );
  return 1;
}

status create_room()
{
  if (current == -1)
  {
    my_notify_fail( "You are not working on any drawing.\n" );
    return 0;
  }
  TP->set_inhibit_graphics( 0 );
  if (!TP->has_client())
    return 1;
  TP->set_inhibit_graphics( 1 );
  if (!room)
  {
    room = clone_object( "/obj/xthing" );
    room->get_unique_idstr();
  }
  room->set_graph( create_graph( ({ ({ background_color, 0, 0, 0, 255, 255,
				       255, 255, 0 }) })
				+ drawings[current][DRAWING_POLYGONS] ) );
  write( room->define() );
  write( "@SCE" + room->query_idstr() + ".@RFS" );
  return 1;
}

status create_something( string str )
{
  if (!stringp( str ))
    return 0;
  if (current == -1)
  {
    my_notify_fail( "You are not working on any drawing.\n" );
    return 0;
  }
  str = lower_case( str );
  if (str == "object")
    return create_object();
/*  else if (str == "room")
    return create_room();*/
  return 0;
}

status bounding_box( string str )
{
  int number, c, c1, minx, miny, maxx, maxy;
  string tmp;

  if (!stringp( str ) || !sscanf( str, "%s %s", tmp, str ) || tmp != "box")
  {
    my_notify_fail( "Usage: bounding box [drawing].\n" );
    return 0;
  }
  if (!stringp( str ) || str == "")
  {
    if (current != -1)
      number = current;
    else
    {
      my_notify_fail( "You are not working on any drawing.\n" );
      return 0;
    }
  }
  else if (sscanf( str, "%d", number ) != 1)
  {
    number = find_drawing( str );
    if (number == -1)
    {
      my_notify_fail( "No drawing is named " + str + ".\n" );
      return 0;
    }
  }
  if (number > sizeof( drawings ) - 1)
  {
    my_notify_fail( "There are not that many drawings.\n" );
    return 0;
  }
  maxx = minx = maxy = miny = -1;
  for (c=0; c < sizeof( drawings[number][DRAWING_POLYGONS] ); c++)
    for (c1=1; c1 < sizeof( drawings[number][DRAWING_POLYGONS][c] ); c1+=2)
    {
      if (drawings[number][DRAWING_POLYGONS][c][c1] > maxx || maxx == -1)
	maxx = drawings[number][DRAWING_POLYGONS][c][c1];
      if (drawings[number][DRAWING_POLYGONS][c][c1] < minx || minx == -1)
	minx = drawings[number][DRAWING_POLYGONS][c][c1];
      if (drawings[number][DRAWING_POLYGONS][c][c1+1] > maxy || maxy == -1)
	maxy = drawings[number][DRAWING_POLYGONS][c][c1+1];
      if (drawings[number][DRAWING_POLYGONS][c][c1+1] < miny || miny == -1)
	miny = drawings[number][DRAWING_POLYGONS][c][c1];
    }
  change_current( number );
  my_write( "Bounding_box = (" + minx + "," + miny + ")-(" + maxx + "," + maxy
	    + "), size = (" + (maxx - minx) + "," + (maxy - miny) + ").\n" );
  return 1;
}

status background( string str )
{
  int colnr;
  
  if (!stringp( str ))
    FAIL( BACKGROUND_HELP );
  if (sscanf( str, "%d", colnr ) == 1)
    if (colnr < 0 || colnr >= 16)
      FAIL( "The colornumber must be between 0 and 15.\n" );
    else;
  else if ((colnr = color_number( str )) == -1)
    FAIL( "There is no color named " + str + ".\n" );
  background_color = colnr;
  if (mode)
    create_room();
  return 1;
}

status flip( string str )
{
  int c, c1;

  if (current == -1)
    FAIL( "You are not working on any drawing.\n" );
  if (!stringp( str ))
    FAIL( "Usage: " + FLIP_HELP );
  str = lower_case( str );
  if (str == "horisontal")
    for (c=0; c < sizeof( drawings[current][DRAWING_POLYGONS]); c++)
      for (c1=1; c1 < sizeof( drawings[current][DRAWING_POLYGONS][c] ); c1+=2)
	drawings[current][DRAWING_POLYGONS][c][c1]
	  = 255 - drawings[current][DRAWING_POLYGONS][c][c1];
  else if (str == "vertical")
    for (c=0; c < sizeof( drawings[current][DRAWING_POLYGONS]); c++)
      for (c1=2; c1 < sizeof( drawings[current][DRAWING_POLYGONS][c] ); c1+=2)
	drawings[current][DRAWING_POLYGONS][c][c1]
	  = 255 - drawings[current][DRAWING_POLYGONS][c][c1];
  else
    FAIL( "Usage: " + FLIP_HELP );
  if (mode)
    create_room();
  return 1;
}

status mode( string str )
{
  if (!stringp( str ))
  {
    my_notify_fail( MODE_HELP );
    return 0;
  }
  str = lower_case( str );
  switch( str )
  {
/*   case "room":
    mode = MODE_ROOM;
    create_room();
    return 1;*/
   case "normal":
    mode = MODE_NORMAL;
    TP->set_inhibit_graphics( 0 );
    ENV( TP )->contents();
    return 1;
   case "paper":
    mode = MODE_PAPER;
    create_room();
    return 1;
   default:
    my_notify_fail( MODE_HELP );
    return 0;
  }
}

string short()
{
  return "a pen";
}

string long()
{
  return "A pen that can be used to create objects and rooms.\n";
}

status id( string str )
{
  return stringp( str ) && (lower_case( str ) == "pen"
			    || str == "Wing's magical pen's unique id");
}

status get()
{
  return 1;
}

status drop()
{
  return 1;
}

string get_drawing( string drawing )
{
  int tmp;
  
  if (!stringp( drawing ) || drawing == ""
      || (tmp = find_drawing( drawing )) == -1)
    return "";
  else
  return create_graph( ({ ({ 7, 0, 0, 0, 255, 255, 255, 255, 0 }) })
		      + drawings[tmp][DRAWING_POLYGONS] );
}

